use crate::error::Result;
use pyo3::PyAny;

pub trait PyAnyExtras {
    fn qualified_class_name(&self) -> Result<String>;
    fn get_proto_meta(&self) -> Result<&PyAny>;
    fn create_instance(&self, field_name: &str) -> Result<&PyAny>;
    fn is_list_field(&self, field_name: &str) -> Result<bool>;
    fn oneof_group(&self, field_name: &str) -> Result<Option<String>>;
}

impl PyAnyExtras for PyAny {
    fn qualified_class_name(&self) -> Result<String> {
        let class = self.getattr("__class__")?;
        let module = class.getattr("__module__")?;
        let name = class.getattr("__name__")?;
        Ok(format!("{module}.{name}"))
    }

    fn get_proto_meta(&self) -> Result<&PyAny> {
        Ok(self.getattr("_betterproto")?)
    }

    fn create_instance(&self, field_name: &str) -> Result<&PyAny> {
        let res = self
            .getattr("cls_by_field")?
            .get_item(field_name)?
            .call0()?;
        Ok(res)
    }

    fn is_list_field(&self, field_name: &str) -> Result<bool> {
        let cls = self.getattr("default_gen")?.get_item(field_name)?;
        let module = cls.getattr("__module__")?;
        let name = cls.getattr("__name__")?;
        Ok(module.to_string() == "builtins" && name.to_string() == "list")
    }

    fn oneof_group(&self, field_name: &str) -> Result<Option<String>> {
        let opt = self
            .getattr("oneof_group_by_field")?
            .call_method1("get", (field_name,))?
            .extract()?;
        Ok(opt)
    }
}
